from httpx._multipart import MultipartStream, get_multipart_boundary_from_content_type

from .prefetch import aprefetch_iterator


async def build_async_multipart_stream(data, files, content_type):
    multipart = MultipartStream(
        data=data or {},
        files=files,
        boundary=get_multipart_boundary_from_content_type(content_type or None),
    )
    return multipart.get_headers(), aprefetch_iterator(multipart)



import asyncio
import time

from .asyncio import run_in_threadpool
from .utils import DummyStopIteration, next_without_stop_iteration


async def to_aiterator(iterator):
    iterator = await run_in_threadpool(iter, iterator)
    while True:
        try:
            yield await run_in_threadpool(next_without_stop_iteration, iterator)
        except DummyStopIteration:
            break


async def rate_limit_iterator(aiterator, iters_per_second):
    start = time.time()
    i = 0
    async for it in aiterator:
        yield it
        await asyncio.sleep((i / iters_per_second) - (time.time() - start))
        i += 1
